 -- optional: create database 
 CREATE DATABASE imoltp 
GO

-- enable for in-memory OLTP - change file path as needed 
 ALTER DATABASE imoltp ADD FILEGROUP imoltp_mod CONTAINS MEMORY_OPTIMIZED_DATA 
 ALTER DATABASE imoltp ADD FILE (name='imoltp_mod1', filename='c:\data\imoltp_mod1') TO FILEGROUP imoltp_mod 
 GO

-- create memory optimized tables 
 USE imoltp 
 GO

-- durable table  contents of this table will not be lost on a server crash 
 CREATE TABLE dbo.ShoppingCart ( 
   ShoppingCartId int not null primary key nonclustered hash with (bucket_count=2000000), 
   UserId int not null index ix_UserId nonclustered hash with (bucket_count=1000000), 
   CreatedDate datetime2 not null, 
   TotalPrice money 
 ) 
 WITH (MEMORY_OPTIMIZED=ON) 
 GO

-- non-durable table  contents of this table are lost on a server restart 
 CREATE TABLE dbo.UserSession ( 
   SessionId int not null primary key nonclustered hash with (bucket_count=400000), 
   UserId int not null, 
   CreatedDate datetime2 not null, 
   ShoppingCartId int, 
   index ix_UserId nonclustered hash (UserId) with (bucket_count=400000) 
 ) 
 WITH (MEMORY_OPTIMIZED=ON, DURABILITY=SCHEMA_ONLY) 
 GO

-- Basic DML 
-- insert a few rows 
 INSERT dbo.UserSession VALUES (1,342,GETUTCDATE(),4) 
 INSERT dbo.UserSession VALUES (2,65,GETUTCDATE(),NULL) 
 INSERT dbo.UserSession VALUES (3,8798,GETUTCDATE(),1) 
 INSERT dbo.UserSession VALUES (4,80,GETUTCDATE(),NULL) 
 INSERT dbo.UserSession VALUES (5,4321,GETUTCDATE(),NULL) 
 INSERT dbo.UserSession VALUES (6,8578,GETUTCDATE(),NULL) 
 INSERT dbo.ShoppingCart VALUES (1,8798,GETUTCDATE(),NULL) 
 INSERT dbo.ShoppingCart VALUES (2,23,GETUTCDATE(),45.4) 
 INSERT dbo.ShoppingCart VALUES (3,80,GETUTCDATE(),NULL) 
 INSERT dbo.ShoppingCart VALUES (4,342,GETUTCDATE(),65.4) 
 GO

-- verify table contents 
 SELECT * FROM dbo.UserSession 
 SELECT * FROM dbo.ShoppingCart 
 GO

 -- update statistics on memory optimized tables 
 UPDATE STATISTICS dbo.UserSession WITH FULLSCAN, NORECOMPUTE 
 UPDATE STATISTICS dbo.ShoppingCart WITH FULLSCAN, NORECOMPUTE 
 GO


 -- in an explicit transaction, assign a cart to a session and update the total price. 
 -- note that the isolation level hint is required for memory-optimized tables with 
 -- SELECT/UPDATE/DELETE statements in explicit transactions 
 BEGIN TRAN 
  UPDATE dbo.UserSession WITH (SNAPSHOT) SET ShoppingCartId=3 WHERE SessionId=4 
  UPDATE dbo.ShoppingCart WITH (SNAPSHOT) SET TotalPrice=65.84 WHERE ShoppingCartId=3 
 COMMIT 
 GO 
 -- verify table contents 
 SELECT * 
 FROM dbo.UserSession u JOIN dbo.ShoppingCart s on u.ShoppingCartId=s.ShoppingCartId 
 WHERE u.SessionId=4 
 GO


-- natively compiled stored procedure for assigning a shopping cart to a session 
 CREATE PROCEDURE dbo.usp_AssignCart @SessionId int 
 WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER 
 AS 
 BEGIN ATOMIC 
 WITH (TRANSACTION ISOLATION LEVEL = SNAPSHOT, LANGUAGE = N'us_english')

  DECLARE @UserId int, 
    @ShoppingCartId int

  SELECT @UserId=UserId, @ShoppingCartId=ShoppingCartId 
  FROM dbo.UserSession WHERE SessionId=@SessionId

  IF @UserId IS NULL 
    THROW 51000, 'The session or shopping cart does not exist.', 1

  UPDATE dbo.UserSession SET ShoppingCartId=@ShoppingCartId WHERE SessionId=@SessionId 
 END 
 GO

 EXEC usp_AssignCart 1 
 GO

 -- natively compiled stored procedure for inserting a large number of rows 
 --   this demonstrates the performance of native procs 
 CREATE PROCEDURE dbo.usp_InsertSampleCarts @StartId int, @InsertCount int 
 WITH NATIVE_COMPILATION, SCHEMABINDING, EXECUTE AS OWNER 
 AS 
 BEGIN ATOMIC 
 WITH (TRANSACTION ISOLATION LEVEL = SNAPSHOT, LANGUAGE = N'us_english')

  DECLARE @ShoppingCartId int = @StartId

  WHILE @ShoppingCartId < @StartId + @InsertCount 
  BEGIN 
    INSERT INTO dbo.ShoppingCart VALUES 
         (@ShoppingCartId, 1, '2013-01-01T00:00:00', NULL) 
    SET @ShoppingCartId += 1 
  END

END 
 GO

-- insert 1,000,000 rows 
 DECLARE @StartId int = (SELECT MAX(ShoppingCartId)+1 FROM dbo.ShoppingCart) 
 EXEC usp_InsertSampleCarts @StartId, 1000000 
 GO

-- verify the rows have been inserted 
 SELECT COUNT(*) FROM dbo.ShoppingCart 
 GO
